import React, { Component } from 'react';
import * as PropTypes from 'prop-types';
import { AgGridReact, AgGridColumn } from 'ag-grid-react';
import DataTableCell from '../DataTableCell/DataTableCell';
import DataTableColumnHeader from '../DataTableColumnHeader/DataTableColumnHeader';
import DataTableFloatingFilter from '../DataTableFloatingFilter/DataTableFloatingFilter';

import 'ag-grid-community/dist/styles/ag-grid.css';
import 'ag-grid-community/dist/styles/ag-theme-material.css';
import './DataTable.css';
import getStyles from './DataTable.styles';

const THEME = 'material';
const SORT_DIRECTIONS = {
    NONE: '',
    ASC: 'asc',
    DESC: 'desc',
};
const FILTER_TYPES = {
    BOOLEAN: 'BOOLEAN',
    DATE: 'DATE',
    DATETIME: 'DATETIME',
    DECIMAL: 'DECIMAL',
    INTEGER: 'INTEGER',
    STRING: 'STRING',
    TIME: 'TIME',
    YEAR: 'YEAR',
};

// TODO: multi-column sort supported

// @TODO Need a right click handler
// @TODO Need paging props

class DataTable extends Component {
    constructor(props) {
        super(props);

        // Init hack to force re-render of ag-grid-react component
        // when props change
        // @see https://github.com/ag-grid/ag-grid-react/issues/77#issuecomment-314717545
        this.key = 0;
    }

    render() {
        this.key += 1;
        const {
            cellRenderer,
            cols,
            contextStyles,
            filterableColumns,
            floatingFilterComponent: FloatingFilterComponent,
            multiSelectMode,
            onApplyFilters,
            onColumnPinned,
            onFilterRequested,
            onSortRequested,
            resizableColumns,
            rows,
            showFilters,
            sortableColumns,
            sortDirection,
            suppressMovableColumns,
            testID,
        } = this.props;

        console.log('DATA TABLE: RENDER: ', this.props); // eslint-disable-line

        const styles = getStyles(contextStyles);

        // Generate container props
        const containerProps = {
            style: styles.container,
            className: 'c-data-table__container',
        };
        if (testID) { containerProps['data-test-id'] = `${testID}__data-table__container`; }

        // Generate viewport props
        const viewportProps = {
            style: styles.viewport,
            className: `ag-theme-${THEME} c-data-table__viewport`,
        };
        if (testID) { containerProps['data-test-id'] = `${testID}__data-table__viewport`; }

        if (FloatingFilterComponent) {
            console.log('USING CUSTOM FLOATING FILTER' ); // eslint-disable-line
        }

        // Generate data table props
        const dataTableProps = {
            reactNext: true,
            floatingFiltersHeight: 110,
            key: this.key,
            suppressColumnVirtualisation: true,
            rowBuffer: 10,
            columnDefs: cols,
            defaultColDef: {
                cols,
                cellRendererFramework: cellRenderer || DataTableCell,
                columnStyles: styles.column,
                filter: filterableColumns,
                floatingFilterComponentFramework: FloatingFilterComponent || DataTableFloatingFilter,
                floatingFilterComponentParams: {
                    filterTypes: FILTER_TYPES,
                    onApplyFilters: (field, value) => {
                        console.log('DATA TABLE: FILTER CHANGED: FIELD: ', field); // eslint-disable-line
                        console.log('DATA TABLE: FILTER CHANGED: VALUE: ', value); // eslint-disable-line
                        if (onApplyFilters) {
                            onApplyFilters(field, value);
                        }
                    },
                    suppressFilterButton: true,
                },
                headerComponentFramework: DataTableColumnHeader,
                headerComponentParams: {
                    sortDirection,
                    sortDirections: SORT_DIRECTIONS,
                    filterableColumns,
                    showFilters,
                    onSortChanged: (field, sortDirection) => {
                        console.log('DATA TABLE: SORT CHANGED: FIELD: ', field); // eslint-disable-line
                        console.log('DATA TABLE: SORT CHANGED: DIRECTION: ', sortDirection); // eslint-disable-line
                        if (onSortRequested) {
                            onSortRequested(field, sortDirection);
                        }
                    },
                    onFilterToggle: (field, showFilter) => {
                        console.log('DATA TABLE: FILTER TOGGLE: FIELD: ', field); // eslint-disable-line
                        console.log('DATA TABLE: FILTER TOGGLE: SHOW FILTER: ', showFilter); // eslint-disable-line
                        if (onFilterRequested) {
                            onFilterRequested(field, showFilter);
                        }
                    },
                    onPinToggle: (field, pin) => {
                        console.log('DATA TABLE: PIN CHANGED: FIELD: ', field); // eslint-disable-line
                        console.log('DATA TABLE: PIN CHANGED: PIN: ', pin); // eslint-disable-line
                        if (onColumnPinned) {
                            onColumnPinned(field, pin);
                        }
                    },
                },
                suppressMenu: true,
                suppressMovable: suppressMovableColumns,
                resizable: resizableColumns,
                sortable: sortableColumns,
            },
            floatingFilter: filterableColumns && showFilters,
            onGridReady: this.handleGridReady,
            onRowClicked: this.handleRowClicked,
            onRowDoubleClicked: this.handleRowDoubleClicked,
            rowData: rows,
            rowSelection: multiSelectMode ? 'multiple' : 'single',
            suppressCellSelection: true,
        };

        console.log('DATA TABLE: RENDER: TABLE PROPS: ', dataTableProps); // eslint-disable-line

        return (
            <div { ...containerProps }>
                <div { ...viewportProps }>
                    <AgGridReact { ...dataTableProps }>
                        { this.renderColumns(cols) }
                    </AgGridReact>
                </div>
            </div>
        );
    }

    /**
     * Renders columns
     * @param {Object} columnList - meta data about columns
     * @returns {Array}
     */
    renderColumns = (columnList) => (
        columnList.map((column) => (
            <AgGridColumn
                autoHeight
                field={ column.field }
                filter={ column.filter }
                headerName={ column.headerName }
                key={ column.field }
                pinned={ column.pinned }
                resizable={ column.resizable }
                sortable={ column.sortable }
                suppressMovable={ column.suppressMovable }
                width={ column.width } />
        ))
    );

    /** Makes grid api available to component */
    handleGridReady = (grid) => {
        this.api = grid.api;
    };

    /**
     * Called on click of a row
     * @returns {void}
     */
    handleRowClicked = () => {
        const { onRowClick } = this.props;

        if (onRowClick) {
            onRowClick({
                data: this.api.getSelectedRows(),
                nodes: this.api.getSelectedNodes(),
            });
        }
    };

    /**
     * Called on double click of a row
     * @returns {void}
     */
    handleRowDoubleClicked = () => {
        const { onRowDoubleClick } = this.props;

        if (onRowDoubleClick) {
            onRowDoubleClick({
                data: this.api.getSelectedRows()[0],
                node: this.api.getSelectedNodes()[0],
            });
        }
    };
}

DataTable.propTypes = {
    /**
     * Custom cell renderer
     * @param {Object} params column details for the cell
     */
    cellRenderer: PropTypes.func,

    /** Array containing column definition object */
    cols: PropTypes.arrayOf(PropTypes.shape({
        /** Unique column id which maps to row property */
        field: PropTypes.string,

        /** Enable / disable individual filterable column (overrides global) */
        filter: PropTypes.bool,

        /** Column header display text */
        headerName: PropTypes.string,

        /** Enables column to be pinned */
        pinned: PropTypes.bool,

        /** Enable / disable individual resizable column (overrides global) */
        resizable: PropTypes.bool,

        /** Enable / disable individual sortable column (overrides global) */
        sortable: PropTypes.bool,

        /** Enable / disable individual movable column (overrides global) */
        suppressMovable: PropTypes.bool,

        /** Width of column */
        width: PropTypes.number,

        // TODO
        type: PropTypes.oneOf([
            FILTER_TYPES.BOOLEAN,
            FILTER_TYPES.DATE,
            FILTER_TYPES.DATETIME,
            FILTER_TYPES.DECIMAL,
            FILTER_TYPES.INTEGER,
            FILTER_TYPES.STRING,
            FILTER_TYPES.TIME,
        ]),
    })).isRequired,

    /** Styles for this component */
    contextStyles: PropTypes.shape({
        /** Column level styles */
        column: PropTypes.object,

        /** Style object for container */
        container: PropTypes.object,

        /** Style object for evenRows*/
        evenRows: PropTypes.object,

        /** Style object for grid filter dialog */
        filter: PropTypes.object,

        /** Style object for grid filter dialog container */
        filterContainer: PropTypes.object,

        /** Style object for grid */
        grid: PropTypes.object,

        /** Style object for header cell */
        headerCell: PropTypes.object,

        /** Style object for header text */
        headerText: PropTypes.object,

        /** Style object for oddRows*/
        oddRows: PropTypes.object,

        /** Style object for grid row */
        row: PropTypes.object,

        /** Style object for selected grid row */
        rowSelected: PropTypes.object,
    }),

    /**
     * Enable / disable global filterable columns (floating filter)
     * Shows filter icon buttons on column headers to toggle filter visibility
     */
    filterableColumns: PropTypes.bool,

    /** Custom floating filter */
    floatingFilterComponent: PropTypes.func,

    /** Enables multiple row selection */
    multiSelectMode: PropTypes.bool,

    // TODO: implement to call server for data refresh
    /** Called after application of filters */
    onApplyFilters: PropTypes.func,

    // TODO: implement to save column position after data refresh
    /** Called after moving of columns */
    // onColumnMoved: PropTypes.func,

    // TODO: implement to save column pinned state after data refresh
    /**
     * Called after pinning of columns
     * @param {String} field
     * @param {Boolean} pin
     */
    onColumnPinned: PropTypes.func,

    // TODO: implement to save column resized state after data refresh
    /** Called after resize of columns */
    // onColumnResized: PropTypes.func,

    /**
     * //
     */
    onFilterRequested: PropTypes.func,

    /**
     * Called on row click
     * @param { { data: Array<Object>, nodes: Array<Object> } }
     */
    onRowClick: PropTypes.func,

    /**
     * Called on row double click
     * @param { { data: Object, node: Object } }
     */
    onRowDoubleClick: PropTypes.func,

    /**
     * Called after sort request
     * @param {String} field
     * @param {String} sortMethod
     * */
    onSortRequested: PropTypes.func,

    /** Enable / disable global resizable columns */
    resizableColumns: PropTypes.bool,

    /** Array of objects of column fields */
    rows: PropTypes.arrayOf(PropTypes.object).isRequired,

    /** Enable / disable visibility of the floating filter (if filterableColumns is enabled) */
    showFilters: PropTypes.bool,

    /**
     * Enable / disable global sortable columns
     * Shows sort icon buttons on column headers to request sorting for the field
     */
    sortableColumns: PropTypes.bool,

    /** Current sort directive */
    sortDirection: PropTypes.oneOf([
        SORT_DIRECTIONS.NONE,
        SORT_DIRECTIONS.ASC,
        SORT_DIRECTIONS.DESC,
    ]),

    /** Enable / disable global movable columns */
    suppressMovableColumns: PropTypes.bool,

    /** ID used for testing */
    testID: PropTypes.string,
};

DataTable.defaultProps = {
    sortDirection: '',
};

export default DataTable;
export {
    SORT_DIRECTIONS,
    FILTER_TYPES,
};
