import React, { useState, useEffect } from 'react';
import * as PropTypes from 'prop-types';
import MUIMenu from '@material-ui/core/Menu';

import { makeStyles } from '@material-ui/core/styles';
import getStyles from './Menu.styles';

/**
 * A Menu displays a list of choices on a temporary surface. It appears when the user interacts with a button, or other control.
 * @see https://material-ui.com/components/menus/
 */
const Menu = (props) => {
    const {
        button,
        children,
        contextStyles,
        open,
        testID,
    } = props;
    const [
        anchorEl,
        setAnchor,
    ] = useState(null);
    const [
        isOpen,
        setOpen,
    ] = useState(open);
    useEffect(() => {
        setOpen(open);
    }, [ open ]);

    // Override core styles with context styles, separating MUI styles
    const {
        container: containerStyles,
        ...muiStyles
    } = getStyles(contextStyles);

    // Create dynamic class names and injected DOM styles for MUI component
    // * makeStyles returns a function
    const styles = makeStyles(muiStyles)();

    // Generate container props
    const containerProps = {
        style: containerStyles,
        className: 'c-menu__trigger',
    };
    if (testID) { containerProps['data-test-id'] = `${testID}__menu__trigger`; }

    // Generate menu props
    const menuProps = {
        classes: {
            paper: styles.menu,
        },
        className: 'c-menu__container',
        anchorEl,
        keepMounted: true,
        open: isOpen,
        onClose: () => { setOpen(false); },
    };
    if (testID) { menuProps['data-test-id'] = `${testID}__menu__container`; }

    // Inject the ability to close the menu on item click
    const kids = React.Children.map(children, (child) => {
        const {
            children: itemChildren,
            onClick,
        } = child.props;
        return React.cloneElement(
            child,
            {
                onClick: (...args) => {
                    if (onClick) {
                        onClick(...args);
                    }
                    setOpen(false);
                },
            },
            itemChildren,
        );
    });

    // Inject the ability to open the menu on button click
    const {
        children: buttonChildren,
        onClick: buttonClick,
    } = button.props;
    const Trigger = React.cloneElement(
        button,
        {
            onClick: (event, ...args) => {
                setAnchor(event.currentTarget);
                if (buttonClick) {
                    buttonClick(event, ...args);
                }
                setOpen(true);
            },
        },
        buttonChildren,
    );

    return (
        <div
            ref={ (r) => setAnchor(r) }
            { ...containerProps }>
            { Trigger }
            <MUIMenu { ...menuProps }>
                { kids }
            </MUIMenu>
        </div>
    );
};

Menu.propTypes = {
    /** Button element that is used for triggering Menu */
    button: PropTypes.node.isRequired,

    /** Menu Items that needs to be displayed inside the Menu */
    children: PropTypes.oneOfType([
        PropTypes.node,
        PropTypes.arrayOf(PropTypes.node),
    ]).isRequired,

    /** Styles for this component */
    contextStyles: PropTypes.shape({
        /** Styles for the container surrounding menu and button */
        container: PropTypes.object,

        /** Styles for the menu container */
        menu: PropTypes.object,
    }),

    /** Sets menu visibility */
    open: PropTypes.bool,

    /** Id used for testing */
    testID: PropTypes.string,
};

Menu.defaultProps = {
    contextStyles: {},
    open: false,
};

export default Menu;
