import React from 'react';
import * as PropTypes from 'prop-types';
import Modal from '../Modal/Modal';
import TextLabel from '../TextLabel/TextLabel';
import ActivityIndicator from '../ActivityIndicator/ActivityIndicator';

import getStyles from './PageActivityIndicator.styles';

/**
 * A control that indicates to the user that a page level action is currently being performed with a loading text and an activity indicator.
 * @see https://material-ui.com/demos/dialogs/
 * @see https://material-ui.com/components/progress/
 */
const PageActivityIndicator = (props) => {
    const {
        contextStyles,
        indicatorSize,
        indicatorThickness,
        isOpen,
        testID,
    } = props;

    const styles = getStyles(contextStyles);

    // Generate modal props
    const modalProps = {
        contextStyles: {
            modal: styles.modal,
            overlay: styles.overlay,
        },
        isModalOpen: isOpen,
        isClosable: false,
    };
    if (testID) { modalProps.testID = `${testID}__page-activity-indicator__modal`; }

    // Generate text props
    const textProps = {
        contextStyles: {
            container: styles.textContainer,
            text: styles.text,
        },
    };
    if (testID) { textProps.testID = `${testID}__page-activity-indicator__text`; }

    // Generate indicator props
    const indicatorProps = {
        contextStyles: {
            container: styles.indicatorContainer,
            indicator: styles.indicator,
        },
        size: indicatorSize,
        thickness: indicatorThickness,
    };
    if (testID) { indicatorProps.testID = `${testID}__page-activity-indicator__indicator`; }

    return (
        <Modal { ...modalProps }>
            <TextLabel { ...textProps }>
                One moment please as the page is being constructed...
            </TextLabel>
            <ActivityIndicator { ...indicatorProps } />
        </Modal>
    );
};

PageActivityIndicator.propTypes = {
    /** Styles for this component */
    contextStyles: PropTypes.shape({
        /** Styles for spinning indicator */
        indicator: PropTypes.object,

        /** Styles for spinning indicator */
        indicatorContainer: PropTypes.object,

        /** Styles for the modal container */
        modal: PropTypes.object,

        /** Styles for the overlay */
        overlay: PropTypes.object,

        /** Styles for loading text */
        text: PropTypes.object,

        /** Styles for loading text */
        textContainer: PropTypes.object,
    }),

    /** Width and Height Of the Activity Indicator in px */
    indicatorSize: PropTypes.number,

    /** Thickness Of the Activity Indicator */
    indicatorThickness: PropTypes.number,

    /** Defines open or closed state of page activity indicator */
    isOpen: PropTypes.bool.isRequired,

    /** Id used for testing */
    testID: PropTypes.string,
};

PageActivityIndicator.defaultProps = {
    contextStyles: {},
    indicatorSize: 75,
    indicatorThickness: 5,
};

export default PageActivityIndicator;
